@@ -67,9 +67,8 @@
 /* Make autoheader happy */
 #undef WITH_SYMBOL_UNDERSCORE
 
-/* Do we have rc4 available? */
-#undef WITH_CMU_RC4
-#undef WITH_SSL_RC4
+/* should we use the internal rc4 library? */
+#undef WITH_RC4
 
 /* do we have des available? */
 #undef WITH_DES
@@ -78,6 +77,9 @@
 /* Do we have kerberos for plaintext password checking? */
 #undef HAVE_KRB
 
+/* do we have SIA for plaintext password checking? */
+#undef HAVE_SIA
+
 /* do we have PAM for plaintext password checking? */
 #undef HAVE_PAM
 
@@ -105,6 +107,12 @@
 /* where do we look for the pwcheck daemon? */
 #undef PWCHECKDIR
 
+/* should we include support for the saslauth daemon? */
+#undef HAVE_SASLAUTHD
+
+/* where does saslauthd look for the communication socket? */
+#undef PATH_SASLAUTHD_RUNDIR
+
 /* do we pay attention to IP addresses in the kerberos 4 tickets? */
 #undef KRB4_IGNORE_IP_ADDRESS
 
