@@ -56,6 +56,7 @@
 #ifdef HAVE_KRB
 #include <krb.h>
 #endif
+
 #include <stdlib.h>
 
 #ifndef WIN32
@@ -87,15 +88,15 @@
 #include <security/pam_appl.h>
 #endif
 
-#ifdef HAVE_PWCHECK
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/uio.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#if defined(HAVE_PWCHECK) || defined(HAVE_SASLAUTHD)
+# include <errno.h>
+# include <sys/types.h>
+# include <sys/uio.h>
+# include <sys/socket.h>
+# include <sys/un.h>
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# endif
 
 extern int errno;
 #endif
@@ -339,6 +340,173 @@ static int kerberos_verify_password(sasl_conn_t *conn,
 
 #endif /* HAVE_KRB */
 
+#ifdef HAVE_GSSAPI_H 
+/* ok, this is  wrong but the most convenient way of doing 
+ * it for now. We assume (possibly incorrectly) that if GSSAPI exists then 
+ * the Kerberos 5 headers and libraries exist.   
+ * What really should be done is a configure.in check for krb5.h and use 
+ * that since none of this code is GSSAPI but rather raw Kerberos5.
+ *
+ * This function also has a bug where an alternate realm can't be
+ * specified.  
+ */
+
+#include <krb5.h>
+
+/* returns 0 for failure, 1 for success */
+static int k5support_verify_tgt(krb5_context context, 
+				krb5_ccache ccache) 
+{
+  krb5_principal server;
+  krb5_data packet;
+  krb5_keyblock *keyblock = NULL;
+  krb5_auth_context auth_context = NULL;
+  krb5_error_code k5_retcode;
+  char thishost[BUFSIZ];
+  int result = 0;
+
+  if (krb5_sname_to_principal(context, NULL, NULL,
+			      KRB5_NT_SRV_HST, &server)) {
+    return 0;
+  }
+
+  if (krb5_kt_read_service_key(context, NULL, server, 0,
+			       0, &keyblock)) {
+    goto fini;
+  }
+
+  if (keyblock) {
+    free(keyblock);
+  }
+
+  /* this duplicates work done in krb5_sname_to_principal
+   * oh well.
+   */
+  if (gethostname(thishost, BUFSIZ) < 0) {
+    goto fini;
+  }
+  thishost[BUFSIZ-1] = '\0';
+  
+  krb5_data_zero(&packet);
+  k5_retcode = krb5_mk_req(context, &auth_context, 0, "host", 
+			   thishost, NULL, ccache, &packet);
+
+  if (auth_context) {
+    krb5_auth_con_free(context, auth_context);
+    auth_context = NULL;
+  }
+
+  if (k5_retcode) {
+    goto fini;
+  }
+
+  if (krb5_rd_req(context, &auth_context, &packet, 
+		  server, NULL, NULL, NULL)) {
+    goto fini;
+  }
+
+  
+  /* all is good now */
+  result = 1;
+ fini:
+  krb5_free_principal(context, server);
+
+  return result;
+}
+
+static int kerberos5_verify_password(sasl_conn_t *conn 
+				     __attribute__((unused)),
+				    const char *user,
+				    const char *passwd,
+				    const char *service 
+				     __attribute__((unused)),
+				    const char *user_realm 
+				               __attribute__((unused)), 
+				    const char **reply)
+{
+  krb5_context context;
+  krb5_ccache ccache = NULL;
+  krb5_principal auth_user;
+  krb5_creds creds;
+  krb5_get_init_creds_opt opts;
+  int result = SASL_FAIL;
+  char tfname[40];
+
+  if (!user|| !passwd) {
+    return SASL_BADPARAM;
+  }
+
+  if (reply) { 
+    *reply = NULL; 
+  }
+  
+  if (krb5_init_context(&context)) {
+    return SASL_FAIL;
+  }
+    
+  if (krb5_parse_name (context, user, &auth_user)) {
+    krb5_free_context(context);
+    return SASL_FAIL;
+  }
+
+  /* create a new CCACHE so we don't stomp on anything */
+  snprintf(tfname,sizeof(tfname), "/tmp/k5cc_%d", getpid());
+  if (krb5_cc_resolve(context, tfname, &ccache)) {
+    krb5_free_principal(context, auth_user);
+    krb5_free_context(context);
+    return SASL_FAIL;
+  }
+
+  if (krb5_cc_initialize (context, ccache, auth_user)) {
+    krb5_free_principal(context, auth_user);
+    krb5_free_context(context);
+    return SASL_FAIL;
+  }
+
+  krb5_get_init_creds_opt_init(&opts);
+  /* 15 min should be more than enough */
+  krb5_get_init_creds_opt_set_tkt_life(&opts, 900); 
+  if (krb5_get_init_creds_password(context, &creds, 
+				   auth_user, passwd, NULL, NULL, 
+				   0, NULL, &opts)) {
+    krb5_cc_destroy(context, ccache);
+    krb5_free_principal(context, auth_user);
+    krb5_free_context(context);
+    return SASL_FAIL;
+  }
+
+  /* at this point we should have a TGT. Let's make sure it is valid */
+  if (krb5_cc_store_cred(context, ccache, &creds)) {
+    krb5_free_principal(context, auth_user);
+    krb5_cc_destroy(context, ccache);
+    krb5_free_context(context);
+    return SASL_FAIL;
+  }
+
+  if (!k5support_verify_tgt(context, ccache)) {
+    goto fini;
+  }
+
+  /* 
+   * fall through -- user is valid beyond this point  
+   */
+  
+  /* try to ensure that no one diddled with result */
+  assert(result == SASL_FAIL); 
+  result = SASL_OK;
+  
+
+ fini:
+/* destroy any tickets we had */
+  krb5_free_cred_contents(context, &creds);
+  krb5_free_principal(context, auth_user);
+  krb5_cc_destroy(context, ccache);
+  krb5_free_context(context);
+ return result;
+}
+
+#endif /* HAVE_GSSAPI_H */
+
 #ifdef HAVE_GETSPNAM
 static int shadow_verify_password(sasl_conn_t *conn __attribute__((unused)),
 				  const char *userid,
@@ -384,8 +552,9 @@ static int sia_verify_password(sasl_conn_t *conn,
 {
   char *host;
   struct sockaddr_in *addr;
-  int argc = 1;
   char *argv[2], *argv0 = "SASL";
+  SIAENTITY *ent;
+  int ret;
   argv[0] = argv0;
   argv[1] = NULL;
 
@@ -401,10 +570,18 @@ static int sia_verify_password(sasl_conn_t *conn,
       host = inet_ntoa(*addr);
 
   /* Try to validate */
-  if (sia_validate_user(NULL, argc, argv, host, userid, NULL, 0, NULL,
-      password) != SIASUCCESS)
+  if (sia_ses_init (&ent, 1, argv, host, userid, NULL, 0, NULL) != SIASUCCESS)
       return SASL_BADAUTH;
-  return SASL_OK;
+  if ((ret = sia_ses_authent (NULL, password, ent)) != SIASUCCESS) {
+      if (ret & SIASTOP)
+          sia_ses_release (&ent);
+      return SASL_BADAUTH;
+  }
+  if (sia_ses_estab (NULL, ent) == SIASUCCESS) {
+      sia_ses_release (&ent);
+      return SASL_OK;
+  }
+  return SASL_BADAUTH;
 }
 #endif /* HAVE_SIA */
 
@@ -516,6 +693,11 @@ static int pam_verify_password(sasl_conn_t *conn __attribute__((unused)),
     if (!userid || !password) {
 	return SASL_BADPARAM;
     }
+    /* protect against a bogus PAM plugin */
+    if (!*userid || !*password) {
+	return SASL_FAIL;
+    }
+
     if (reply) { *reply = NULL; }
 
     my_conv.appdata_ptr = &pd;
@@ -897,11 +1079,118 @@ static int pwcheck_verify_password(sasl_conn_t *conn,
 
 #endif
 
+#ifdef HAVE_SASLAUTHD
+/* saslauthd-authenticated login */
+static int saslauthd_verify_password(sasl_conn_t *conn,
+				   const char *userid, 
+				   const char *passwd,
+				   const char *service __attribute__((unused)),
+				   const char *user_realm 
+				               __attribute__((unused)), 
+				   const char **reply)
+{
+    static char response[1024];
+    int s;
+    struct sockaddr_un srvaddr;
+    int r, n;
+    unsigned int start;
+    sasl_getopt_t *getopt;
+    void *context;
+    char pwpath[sizeof(srvaddr.sun_path)];
+    const char *p = NULL;
+
+    if (reply)
+	*reply = NULL;
+
+    /* check to see if the user configured a rundir */
+    if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context) == SASL_OK) {
+	getopt(context, NULL, "saslauthd_path", &p, NULL);
+    }
+    if (p) {
+	strncpy(pwpath, p, sizeof(pwpath));
+    } else {
+	if (strlen(PATH_SASLAUTHD_RUNDIR) + 4 + 1 > sizeof(pwpath))
+	    return SASL_FAIL;
+
+	strcpy(pwpath, PATH_SASLAUTHD_RUNDIR);
+	strcat(pwpath, "/mux");
+    }
+
+    s = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (s == -1)
+	return errno;
+
+    memset((char *)&srvaddr, 0, sizeof(srvaddr));
+    srvaddr.sun_family = AF_UNIX;
+    strncpy(srvaddr.sun_path, pwpath, sizeof(srvaddr.sun_path));
+
+    r = connect(s, (struct sockaddr *) &srvaddr, sizeof(srvaddr));
+    if (r == -1) {
+	if (reply)
+	    *reply = "cannot connect to pwcheck server";
+	return SASL_FAIL;
+    }
+
+    {
+	int u_len;
+	int p_len;
+	char *msg;
+
+	u_len = strlen(userid) + 1;
+	p_len = strlen(passwd) + 1;
+	msg = sasl_ALLOC(u_len + p_len);
+	if (msg == NULL) {
+	    close(s);
+	    if (reply)
+		*reply = "not enough memory";
+	    return SASL_FAIL;
+	}
+	strcpy(msg, userid);
+	strcpy(msg + u_len, passwd);
+
+	while (write(s, msg, u_len + p_len) == -1)
+	    switch (errno) {
+	    case EINTR:
+		continue;
+	    default:
+		sasl_FREE(msg);
+		if (reply)
+		    *reply = "write failed";
+		return SASL_FAIL;
+	    }
+
+	sasl_FREE(msg);
+    }
+
+    start = 0;
+    while (start < sizeof(response) - 1) {
+	n = read(s, response + start, sizeof(response) - 1 - start);
+	if (n < 1)
+	    break;
+	start += n;
+    }
+
+    close(s);
+
+    if (start > 1 && !strncmp(response, "OK", 2))
+	return SASL_OK;
+
+    response[start] = '\0';
+    if (reply)
+	*reply = response;
+    return SASL_BADAUTH;
+}
+
+#endif
+
 struct sasl_verify_password_s _sasl_verify_password[] = {
     { "sasldb", &sasldb_verify_password },
 #ifdef HAVE_KRB
     { "kerberos_v4", &kerberos_verify_password },
 #endif
+#ifdef HAVE_GSSAPI_H
+    { "kerberos_v5", &kerberos5_verify_password },
+#endif
 #ifdef HAVE_GETSPNAM
     { "shadow", &shadow_verify_password },
 #endif
@@ -916,6 +1205,9 @@ struct sasl_verify_password_s _sasl_verify_password[] = {
 #endif
 #ifdef HAVE_PWCHECK
     { "pwcheck", &pwcheck_verify_password },
+#endif
+#ifdef HAVE_SASLAUTHD
+    { "saslauthd", &saslauthd_verify_password },
 #endif
     { NULL, NULL }
 };
