@@ -1,27 +1,45 @@
 /* SASL server API implementation
  * Tim Martin
  */
-/***********************************************************
-        Copyright 1998 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of CMU not be
-used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
-
-CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-******************************************************************/
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
 #include <config.h>
 #include <stdio.h>
@@ -252,7 +270,7 @@ static void client_done(void) {
       _sasl_done_with_plugin(cprevm->library);
     sasl_FREE(cprevm);    
   }
-  sasl_FREE(cmechlist->mutex);
+  sasl_MUTEX_DISPOSE(cmechlist->mutex);
   _sasl_free_utils(&cmechlist->utils);
   sasl_FREE(cmechlist);
 }
@@ -354,12 +372,15 @@ int sasl_client_init(const sasl_callback_t *callbacks)
   cmechlist->mech_length=0;
 
   add_plugin((void *) &external_client_init, NULL);
-  
-  ret=_sasl_get_mech_list("sasl_client_plug_init",
-			  _sasl_find_getpath_callback(callbacks),
-			  _sasl_find_verifyfile_callback(callbacks),
-			  &add_plugin);
 
+  ret = _sasl_common_init();
+
+  if (ret == SASL_OK)
+      ret=_sasl_get_mech_list("sasl_client_plug_init",
+			      _sasl_find_getpath_callback(callbacks),
+			      _sasl_find_verifyfile_callback(callbacks),
+			      &add_plugin);
+  
   return ret;
 }
 
@@ -419,7 +440,7 @@ int sasl_client_new(const char *service,
 			   &client_idle, NULL,
 			   prompt_supp, &global_callbacks);
   if (result != SASL_OK) return result;
-
+  
   conn = (sasl_client_conn_t *)*pconn;
 
   conn->mech = NULL;
@@ -510,6 +531,7 @@ int sasl_client_start(sasl_conn_t *conn,
     size_t pos=0,place;
     size_t list_len;
     sasl_ssf_t bestssf = 0, minssf = 0;
+    int result;
 
     /* verify parameters */
     if (list == NULL)
@@ -517,7 +539,6 @@ int sasl_client_start(sasl_conn_t *conn,
 	return SASL_BADPARAM;
     }
 
-
     VL(("in sasl_client_start\n"));
 
     /* if prompt_need != NULL we've already been here
@@ -525,13 +546,15 @@ int sasl_client_start(sasl_conn_t *conn,
 
     /* do a step */
     if (prompt_need && *prompt_need != NULL) {
-	return c_conn->mech->plug->mech_step(conn->context,
-					     c_conn->cparams,
-					     NULL,
-					     0,
-					     prompt_need,
-					     clientout, (int *) clientoutlen,
-					     &conn->oparams);
+	result = c_conn->mech->plug->mech_step(conn->context,
+					       c_conn->cparams,
+					       NULL,
+					       0,
+					       prompt_need,
+					       clientout, (int *) clientoutlen,
+					       &conn->oparams);
+
+	return result;
     }
 
     /* set secret */
@@ -617,7 +640,8 @@ int sasl_client_start(sasl_conn_t *conn,
 
     if (bestm == NULL) {
 	VL(("No worthy mechs found\n"));
-	return SASL_NOMECH;
+	result = SASL_NOMECH;
+	goto done;
     }
 
     /* make cparams */
@@ -633,13 +657,15 @@ int sasl_client_start(sasl_conn_t *conn,
 				 &(conn->context));
 
     /* do a step */
-    return c_conn->mech->plug->mech_step(conn->context,
+    result = c_conn->mech->plug->mech_step(conn->context,
 					 c_conn->cparams,
 					 NULL,
 					 0,
 					 prompt_need,
 					 clientout, (int *) clientoutlen,
-					 &conn->oparams);
+					 &conn->oparams);    
+ done:
+    return result;
 }
 
 /* do a single authentication step.
@@ -664,19 +690,21 @@ int sasl_client_step(sasl_conn_t *conn,
 		     unsigned *clientoutlen)
 {
   sasl_client_conn_t *c_conn= (sasl_client_conn_t *) conn;
+  int result;
 
   /* check parameters */
   if ((serverin==NULL) && (serverinlen>0))
     return SASL_BADPARAM;
 
   /* do a step */
-   return c_conn->mech->plug->mech_step(conn->context,
-				    c_conn->cparams,
-				    serverin,
-				    serverinlen,
-				    prompt_need,
-				    clientout, (int *)clientoutlen,
-				    &conn->oparams);
+  result = c_conn->mech->plug->mech_step(conn->context,
+					 c_conn->cparams,
+					 serverin,
+					 serverinlen,
+					 prompt_need,
+					 clientout, (int *)clientoutlen,
+					 &conn->oparams);
+  return result;
 }
 
 /* Set connection secret based on passphrase
