@@ -1,27 +1,45 @@
 /* saslint.h - internal SASL library definitions
  * Tim Martin
  */
-/***********************************************************
-        Copyright 1998 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of CMU not be
-used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
-
-CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-******************************************************************/
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
 #ifndef SASLINT_H
 #define SASLINT_H
@@ -30,10 +48,18 @@ SOFTWARE.
 #include "sasl.h"
 #include "saslplug.h"
 
+extern int _sasl_common_init(void);
+
+/* dlopen.c */
 extern int _sasl_get_mech_list(const char *entryname,
 			       const sasl_callback_t *getpath_callback,
 			       const sasl_callback_t *verifyfile_callback,
 			       int (*add_plugin)(void *,void *));
+extern int _sasl_get_plugin(const char *file,
+			    const char *entryname,
+			    const sasl_callback_t *verifyfile_callback,
+			    void **entrypoint,
+			    void **library);
 
 extern const sasl_callback_t *
 _sasl_find_getpath_callback(const sasl_callback_t *callbacks);
@@ -62,7 +88,6 @@ typedef struct {
 struct sasl_conn {
   void (*destroy_conn)(sasl_conn_t *); /* destroy function */
 
-  int open; /* connection open or not */
   char *service;
 
   int secflags;  /* security layer flags passed to sasl_*_new */
@@ -77,10 +102,6 @@ struct sasl_conn {
 
   sasl_secret_t *secret;
 
-  int uses_sec_layer;   /* if need to encrypt/decrpt all transmissions */
-
-  void *mutex;
-
   int (*idle_hook)(sasl_conn_t *conn);
   const sasl_callback_t *callbacks;
   const sasl_global_callbacks_t *global_callbacks; /* global callbacks
@@ -170,33 +191,19 @@ extern int sasl_config_getint(const char *key,int def);
 extern int sasl_config_getswitch(const char *key,int def);
 
 /* clear password checking declarations (checkpw.c) */
-extern int _sasl_passwd_verify_password(sasl_conn_t *conn,
-					const char *userid,
-					const char *password,
-					const char **reply);
-extern int _sasl_shadow_verify_password(sasl_conn_t *conn,
-					const char *userid,
-					const char *password,
-					const char **reply);
-extern int _sasl_kerberos_verify_password(sasl_conn_t *conn,
-					  const char *user,
-					  const char *passwd,
-					  const char *service,
-					  const char **reply);
-extern int _sasl_PAM_verify_password(sasl_conn_t *conn,
-				     const char *userid,
-				     const char *password, 
-				     const char *service,
-				     const char **reply);
-extern int _sasl_sasldb_verify_password(sasl_conn_t *conn,
-					const char *userid, 
-					const char *passwd,
-					const char *user_realm,
-					const char **reply);
-extern int _sasl_pwcheck_verify_password(sasl_conn_t *conn,
-					 const char *userid,
-					 const char *passwd,
-					 const char **reply);
+typedef int sasl_plaintext_verifier(sasl_conn_t *conn,
+				    const char *userid,
+				    const char *passwd,
+				    const char *service,
+				    const char *user_realm,
+				    const char **reply);
+struct sasl_verify_password_s {
+    char *name;
+    sasl_plaintext_verifier *verify;
+};
+
+extern struct sasl_verify_password_s _sasl_verify_password[];
+
 extern int _sasl_sasldb_set_pass(sasl_conn_t *conn,
 				 const char *user, 
 				 const char *pass,
