@@ -1,27 +1,45 @@
 /* saslutil.c
  * Tim Martin 5/20/98
  */
-/***********************************************************
-        Copyright 1998 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of CMU not be
-used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
-
-CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-******************************************************************/
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
 #include <config.h>
 #include <stdio.h>
@@ -257,16 +275,20 @@ int sasl_utf8verify(const char *str, unsigned len)
 void getranddata(unsigned short ret[3])
 {
     long curtime;
-    FILE *f;
     
-    memset(ret, 0, sizeof(ret));
-    
-    /* this will probably only work on linux */
-    if ((f = fopen(DEV_RANDOM, "r")) != NULL) {
-	fread(ret, 1, sizeof(ret), f);
-	fclose(f);
-	return;
+    memset(ret, 0, sizeof(unsigned short)*3);
+
+#ifdef DEV_RANDOM    
+    {
+        FILE *f;
+
+	if ((f = fopen(DEV_RANDOM, "r")) != NULL) {
+	    fread(ret, 1, sizeof(ret), f);
+	    fclose(f);
+	    return;
+        }
     }
+#endif
 
 #ifdef HAVE_GETPID
     ret[0] = (unsigned short) getpid();
@@ -283,9 +305,8 @@ void getranddata(unsigned short ret[3])
 	    ret[1] ^= (unsigned short) (clock() & 0xFFFF);
 	    ret[1] ^= (unsigned short) (tv.tv_usec >> 16);
 	    ret[2] ^= (unsigned short) (tv.tv_usec & 0xFFFF);
+	    return;
 	}
-
-	return;
     }
 #endif /* HAVE_GETTIMEOFDAY */
     
