@@ -1,28 +1,46 @@
 /* Kerberos4 SASL plugin
  * Tim Martin 
- * $Id: kerberos4.c,v 1.53 2000/02/24 01:36:22 leg Exp $
+ * $Id: kerberos4.c,v 1.65 2001/02/19 19:01:54 leg Exp $
+ */
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/***********************************************************
-        Copyright 1998 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of CMU not be
-used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
-
-CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-******************************************************************/
 
 #include <config.h>
 #include <krb.h>
@@ -44,6 +62,8 @@ SOFTWARE.
 #include <saslutil.h>
 #include <saslplug.h>
 
+#include <errno.h>
+
 #ifdef WIN32
 /* This must be after sasl.h, saslutil.h */
 # include "saslKERBEROSV4.h"
@@ -196,7 +216,7 @@ static int privacy_decode(void *context,
 	    text->size=ntohl(text->size);
 	    
 	    /* too big? */
-	    if (text->size>0xFFFF) return SASL_FAIL;
+	    if ((text->size>0xFFFF) || (text->size < 0)) return SASL_FAIL;
 	    
 	    if (text->bufsize < text->size + 5) {
 		text->buffer = text->realloc(text->buffer, text->size + 5);
@@ -347,7 +367,7 @@ static int integrity_decode(void *context, const char *input, unsigned inputlen,
 	memcpy(&(text->size), text->sizebuf, 4);
 	text->cursize=0;
 	text->size=ntohl(text->size);
-	if (text->size>0xFFFF) return SASL_FAIL; /* too big probably error */
+	if ((text->size>0xFFFF) || (text->size < 0)) return SASL_FAIL; /* too big probably error */
 
 	if (text->bufsize < text->size) {
 	    text->buffer = text->realloc(text->buffer, text->size);
@@ -567,6 +587,12 @@ static int server_continue_step (void *conn_context,
     /* received authenticator */
 
     /* create ticket */
+    if (clientinlen > MAX_KTXT_LEN) {
+	if (errstr)
+	    *errstr = "Request larger than maximum ticket size";
+	return SASL_FAIL;
+    }
+
     ticket.length=clientinlen;
     for (lup=0;lup<clientinlen;lup++)      
       ticket.dat[lup]=clientin[lup];
@@ -655,13 +681,20 @@ static int server_continue_step (void *conn_context,
   {
     int result;
     int testnum;
-    int lup, flag;
-    unsigned char in[1024];
+    int flag;
+    unsigned char *in;
 
-    for (lup=0;lup<clientinlen;lup++)
-      in[lup]=clientin[lup];
+    if ((clientinlen==0) || (clientinlen % 8 != 0)) {
+	if (errstr)
+	    *errstr = "Response to challenge is not a multiple of 8 octets (a DES block)";
+	return SASL_FAIL;	
+    }
 
-    in[lup]=0;
+    /* we need to make a copy because des does in place decrpytion */
+    in = sparams->utils->malloc(clientinlen + 1);
+    if (in == NULL) return SASL_NOMEM;
+    memcpy(in, clientin, clientinlen);
+    in[clientinlen]='\0';
 
     /* decrypt; verify checksum */
 
@@ -788,10 +821,12 @@ static int server_continue_step (void *conn_context,
     text->size=-1;
     text->needsize=4;
 
+    sparams->utils->free(in);
     return SASL_OK;
   }
-
-
+  
+  if (errstr)
+      *errstr = "Improper step. Probably application error";
   return SASL_FAIL; /* should never get here */
 }
 
@@ -824,6 +859,10 @@ int sasl_server_plug_init(sasl_utils_t *utils,
     const char *ret;
     unsigned int rl;
     
+    if (maxversion < KERBEROS_VERSION) {
+	return SASL_BADVERS;
+    }
+
     utils->getopt(utils->getopt_context, "KERBEROS_V4", "srvtab", &ret, &rl);
 
     if (ret == NULL) {
@@ -834,16 +873,16 @@ int sasl_server_plug_init(sasl_utils_t *utils,
     strcpy(srvtab, ret);
 
     /* fail if we can't open the srvtab file */
-    if (access(srvtab, R_OK) != 0)
+    if (access(srvtab, R_OK) != 0) {
+	utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", SASL_FAIL, errno,
+		   "can't access srvtab file %s: %m", srvtab);
+	utils->free(srvtab);
 	return SASL_FAIL;
+    }
 
-    if (maxversion<KERBEROS_VERSION)
-	return SASL_BADVERS;
-
-    *pluglist=plugins;
-
-    *plugcount=1;  
-    *out_version=KERBEROS_VERSION;
+    *pluglist = plugins;
+    *plugcount = 1;
+    *out_version = KERBEROS_VERSION;
     
     return SASL_OK;
 }
@@ -857,6 +896,22 @@ static int client_start(void *glob_context __attribute__((unused)),
   return new_text(params->utils, (context_t **) conn);
 }
 
+static void free_prompts(sasl_client_params_t *params,
+			 sasl_interact_t *prompts)
+{
+    sasl_interact_t *ptr=prompts;
+    if (ptr==NULL) return;
+  
+    do {
+	if (ptr->result!=NULL) params->utils->free(ptr->result);
+      
+	ptr++;
+    } while (ptr->id != SASL_CB_LIST_END);
+  
+    params->utils->free(prompts);
+    prompts = NULL;
+}
+
 static int client_continue_step (void *conn_context,
 				 sasl_client_params_t *params,
 				 const char *serverin,
@@ -866,362 +921,384 @@ static int client_continue_step (void *conn_context,
 				 int *clientoutlen,
 				 sasl_out_params_t *oparams)
 {
-  KTEXT_ST authent;
-  context_t *text;
-  text=conn_context;
+    KTEXT_ST authent;
+    context_t *text;
+    text=conn_context;
 
-  authent.length = MAX_KTXT_LEN;
+    authent.length = MAX_KTXT_LEN;
   
-  if (text->state==0)
-  {
-    VL(("KERBEROS_V4 Step 1\n"));
-
-    *clientout = NULL;
-    *clientoutlen = 0;
-
-    text->state=1;
-
-    return SASL_CONTINUE;
-  }
-
-  if (text->state==1)
-  {
-    /* We should've just recieved a 32-bit number in network byte order.
-     * We want to reply with an authenticator. */
-    int result;
-    KTEXT_ST ticket;
-    char *service=(char *)params->service;
-
-    VL(("KERBEROS_V4 Step 2\n"));
-
-    memset(&ticket, 0L, sizeof(ticket));
-    ticket.length=MAX_KTXT_LEN;   
-
-    if (serverinlen != 4)
+    if (text->state==0)
     {
-      VL(("serverin not 4 bytes long\n"));
-      return SASL_FAIL; 
-    }
+	VL(("KERBEROS_V4 Step 1\n"));
 
-    memcpy(&text->challenge, serverin, 4);
-
-    text->challenge=ntohl(text->challenge); 
-
-    if (params->serverFQDN == NULL) {
-	VL(("No serverFQDN set\n"));
-	return SASL_BADPARAM;
-    }
-    if (params->service == NULL) {
-	VL(("No service set\n"));
-	return SASL_BADPARAM;
-    }
-    if (params->service==NULL)
-    {
-      VL(("No service set\n"));
-      return SASL_BADAUTH;
-    }
-
-    text->realm=krb_realmofhost(params->serverFQDN);
-    text->hostname=(char *) params->serverFQDN;
-
-    /* the instance of the principal we're authenticating with */
-    strncpy (text->instance, krb_get_phost (params->serverFQDN), 
-	     sizeof (text->instance));
-    /* text->instance is NULL terminated unless it was too long */
-    text->instance[sizeof(text->instance)-1] = '\0';
-
-    VL (("service=%s\n", service));
-    VL (("instance=%s\n",text->instance));
-
-    if ((result=krb_mk_req(&ticket, service, text->instance,
-			   text->realm,text->challenge))) 
-    {
-	VL(("krb_mk_req failed service=%s instance=%s realm=%s krb error=%d\n",
-	    service,text->instance,text->realm,result));
-	return SASL_FAIL;
-    }
-    
-    *clientout=params->utils->malloc(ticket.length);
-    memcpy((char *) (*clientout), ticket.dat, ticket.length);
-    *clientoutlen=ticket.length;
-
-    text->state=2;
-    return SASL_CONTINUE;
-  }
-
-  /* challenge #2 */
-  if (text->state==2)
-  {
-    int need = 0;
-    int musthave = 0;
-    int testnum;
-    int nchal;    
-    unsigned char sout[1024];
-    unsigned len;
-    unsigned char in[8];
-    const char *userid;
-    int result;
-    sasl_getsimple_t *getuser_cb;
-    void *getuser_context;
-    sasl_interact_t *prompt;
-    int prompt_for_userid = 0;
-    int servermaxbuf;
-
-    if (prompt_need && *prompt_need) {
-      /* If we requested prompts, make sure they're
-       * properly filled in. */
-      for (prompt = *prompt_need;
-	   prompt->id != SASL_CB_LIST_END;
-	   ++prompt)
-	if (! prompt->result)
-	  return SASL_BADPARAM;
-
-      /* Get the username */
-      if (! oparams->user)
-	for (prompt = *prompt_need;
-	     prompt->id != SASL_CB_LIST_END;
-	     ++prompt)
-	  if (prompt->id == SASL_CB_USER) {
-	    oparams->user
-	      = params->utils->malloc(strlen(prompt->result) + 1);
-	    if (! oparams->user)
-	      return SASL_NOMEM;
-	    strcpy(oparams->user, prompt->result);
-	    break;
-	  }
-
-      params->utils->free(*prompt_need);
-      *prompt_need = NULL;
-    }
-
-    /* Now, try to get the userid by normal means... */
-    if (! oparams->user) {
-      /* Try to get the callback... */
-      result = params->utils->getcallback(params->utils->conn,
-					  SASL_CB_USER,
-					  &getuser_cb,
-					  &getuser_context);
-      switch (result) {
-      case SASL_INTERACT:
-	/* We'll set up an interaction later. */
-	prompt_for_userid = 1;
-	break;
-      case SASL_OK:
-	if (! getuser_cb)
-	  break;
-	result = getuser_cb(getuser_context,
-			    SASL_CB_USER,
-			    &userid,
-			    NULL);
-	if (result != SASL_OK)
-	  return result;
-	if (userid) {
-	  oparams->user = params->utils->malloc(strlen(userid) + 1);
-	  if (! oparams->user)
-	    return SASL_NOMEM;
-	  strcpy(oparams->user, userid);
+	if (clientout) {
+	    *clientout = NULL;
+	    *clientoutlen = 0;
 	}
-	break;
-      default:
-	return result;
-      }
-    }
-      
-    /* And now, if we *still* don't have userid,
-     * but we think we can prompt, we need to set up a prompt. */
-    if (! oparams->user && prompt_for_userid) {
-      if (! prompt_need)
-	return SASL_INTERACT;
-      *prompt_need = params->utils->malloc(sizeof(sasl_interact_t) * 2);
-      if (! *prompt_need)
-	return SASL_NOMEM;
-      prompt = *prompt_need;
-      prompt->id = SASL_CB_USER;
-      prompt->prompt = "Remote Userid";
-      prompt->defresult = NULL;
-      prompt++;
-      prompt->id = SASL_CB_LIST_END;
-      return SASL_INTERACT;
-    }
-      
-    /* must be 8 octets */
-    if (serverinlen!=8)
-    {
-      return SASL_BADAUTH;
+
+	text->state=1;
+
+	return SASL_CONTINUE;
     }
 
-    memcpy(in, serverin, 8);
+    else if (text->state==1) {
+	/* We should've just recieved a 32-bit number in network byte order.
+	 * We want to reply with an authenticator. */
+	int result;
+	KTEXT_ST ticket;
 
-    /* get credentials */
-    if ((krb_get_cred((char *)params->service,
-		      text->instance,
-		      text->realm,
-		      &text->credentials)))
-    {
-      return SASL_BADAUTH;
-    }
+	VL(("KERBEROS_V4 Step 2\n"));
 
-    memcpy(text->session, text->credentials.session, 8);
+	memset(&ticket, 0L, sizeof(ticket));
+	ticket.length=MAX_KTXT_LEN;   
 
-    /* make key schedule for encryption and decryption */
-    des_key_sched(&text->session, text->init_keysched);
-    des_key_sched(&text->session, text->enc_keysched);
-    des_key_sched(&text->session, text->dec_keysched);
+	if (serverinlen != 4) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", SASL_FAIL,
+			       0, "server challenge not 4 bytes long");
+	    return SASL_FAIL; 
+	}
 
-    /* decrypt from server */
-    des_ecb_encrypt((des_cblock *)in, (des_cblock *)in,
-		    text->init_keysched, DES_DECRYPT);
+	memcpy(&text->challenge, serverin, 4);
 
-    /* convert to 32bit int */
-    testnum = (in[0]*256*256*256)+(in[1]*256*256)+(in[2]*256)+in[3];
+	text->challenge=ntohl(text->challenge); 
 
-    /* verify data 1st 4 octets must be equal to chal+1 */
-    if (testnum != text->challenge+1)
-    {
-      VL(("challenge not right\n"));
-      return SASL_BADAUTH;
-    }
+	if (params->serverFQDN == NULL) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADPARAM, 0, 
+			       "no 'serverFQDN' set");
+	    return SASL_BADPARAM;
+	}
+	if (params->service == NULL) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADPARAM, 0, 
+			       "no 'service' set");
+	    return SASL_BADPARAM;
+	}
 
-    /* construct 8 octets
-     * 1-4 - original checksum
-     * 5 - bitmask of sec layer
-     * 6-8 max buffer size
-     */
+	text->realm=krb_realmofhost(params->serverFQDN);
+	text->hostname=(char *) params->serverFQDN;
 
+	/* the instance of the principal we're authenticating with */
+	strncpy (text->instance, krb_get_phost (params->serverFQDN), 
+		 sizeof (text->instance));
+	/* text->instance is NULL terminated unless it was too long */
+	text->instance[sizeof(text->instance)-1] = '\0';
 
-    if (params->props.min_ssf > KRB_DES_SECURITY_BITS + params->external_ssf) {
-      VL (("Minimum ssf too strong min_ssf=%u\n",
-	   params->props.min_ssf));
-      return SASL_TOOWEAK;
-    } else if (params->props.min_ssf > params->props.max_ssf) {
-      return SASL_BADPARAM;
-    }
-
-    /* create stuff to send to server */
-    nchal=htonl(text->challenge);
-    memcpy(sout, &nchal, 4);
-
-    /* need bits of layer */
-    need = params->props.max_ssf - params->external_ssf;
-    musthave = params->props.min_ssf - params->external_ssf;
-
-    VL(("need = %i musthave = %i \n",need, musthave));
-
-    if ((in[4] & KRB_SECFLAG_ENCRYPTION)
-	&& (need>=56) && (musthave <= 56)) {
-      /* encryption */
-      oparams->encode = &privacy_encode;
-      oparams->decode = &privacy_decode;
-      oparams->mech_ssf = 56;
-      sout[4] = KRB_SECFLAG_ENCRYPTION;
-      VL (("Using encryption layer\n"));
-    } else if ((in[4] & KRB_SECFLAG_INTEGRITY)
-	       && (need >= 1) && (musthave <= 1)) {
-      /* integrity */
-      oparams->encode=&integrity_encode;
-      oparams->decode=&integrity_decode;
-      oparams->mech_ssf=1;
-      sout[4] = KRB_SECFLAG_INTEGRITY;
-      VL (("Using integrity layer\n"));
-    } else if ((in[4] & KRB_SECFLAG_NONE) && (musthave <= 0)) {
-      /* no layer */
-      oparams->encode=NULL;
-      oparams->decode=NULL;
-      oparams->mech_ssf=0;
-      sout[4] = KRB_SECFLAG_NONE;
-      VL (("Using no layer\n"));
-    } else {
-      return SASL_BADPROT;
-    }
-
-    servermaxbuf=in[5]*256*256+in[6]*256+in[7];
-    oparams->maxoutbuf=servermaxbuf;
-
-    sout[5] = (oparams->maxoutbuf) >> 16;  /* max ciphertext buffer size */
-    sout[6] = (oparams->maxoutbuf) >> 8;
-    sout[7] = (oparams->maxoutbuf);
-
-    sout[8] = 0x00; /* just to be safe */
-
-    /* append userid */
-    len = 9;			/* 8 + trailing NULL */
-    if (oparams->user) {
-      strcpy((char *)sout + 8, oparams->user);
-      len += strlen(oparams->user);
-    }
-
-    /* append 0 based octets so is multiple of 8 */
-    while(len % 8)
-    {
-      sout[len]=0;
-      len++;
-    }
-    sout[len]=0;
+	if ((result=krb_mk_req(&ticket, (char *) params->service, 
+			       text->instance, text->realm,text->challenge)))
+	{
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_FAIL, 0, 
+			       "krb_mk_req() failed: %s (%d)",
+			       krb_err_txt[result], result);
+	    return SASL_FAIL;
+	}
     
-    des_pcbc_encrypt((des_cblock *)sout,
-		     (des_cblock *)sout,
-		     len,
-		     text->init_keysched,
-		     (des_cblock *)text->session,
-		     DES_ENCRYPT);
+	*clientout=params->utils->malloc(ticket.length);
+	memcpy((char *) (*clientout), ticket.dat, ticket.length);
+	*clientoutlen=ticket.length;
 
-    *clientout = params->utils->malloc(len);
-    memcpy((char *) *clientout, sout, len);
-
-    *clientoutlen=len;
-
-    /* nothing more to do; should be authenticated */
-    result = params->utils->getprop(params->utils->conn,
-                          SASL_IP_LOCAL, (void **)&(text->ip_local));
-    if (result != SASL_OK)
-    {
-	VL(("Unable to get SASL_IP_LOCAL (local IP address)\n"));
-	return result;
+	text->state=2;
+	return SASL_CONTINUE;
     }
 
-    result = params->utils->getprop(params->utils->conn,
-                          SASL_IP_REMOTE, (void **)&(text->ip_remote));
-    if (result != SASL_OK)
-    {
-	VL(("Unable to get SASL_IP_REMOTE (IP address of server we're contacting)\n"));
-	return result;
+    /* challenge #2 */
+    else if (text->state==2) {
+	int need = 0;
+	int musthave = 0;
+	int testnum;
+	int nchal;    
+	unsigned char *sout = NULL;
+	unsigned len;
+	unsigned char in[8];
+	const char *userid;
+	int result;
+	sasl_getsimple_t *getuser_cb;
+	void *getuser_context;
+	sasl_interact_t *prompt;
+	int prompt_for_userid = 0;
+	int servermaxbuf;
+
+	if (prompt_need && *prompt_need) {
+	    /* If we requested prompts, make sure they're
+	     * properly filled in. */
+	    for (prompt = *prompt_need;
+		 prompt->id != SASL_CB_LIST_END;
+		 ++prompt)
+		if (! prompt->result)
+		    return SASL_BADPARAM;
+
+	    /* Get the username */
+	    if (! oparams->user)
+		for (prompt = *prompt_need;
+		     prompt->id != SASL_CB_LIST_END;
+		     ++prompt)
+		    if (prompt->id == SASL_CB_USER) {
+			oparams->user
+			    = params->utils->malloc(strlen(prompt->result) + 1);
+			if (! oparams->user)
+			    return SASL_NOMEM;
+			strcpy(oparams->user, prompt->result);
+			break;
+		    }
+
+	    free_prompts(params, *prompt_need);
+	    *prompt_need = NULL;
+	}
+
+	/* Now, try to get the userid by normal means... */
+	if (! oparams->user) {
+	    /* Try to get the callback... */
+	    result = params->utils->getcallback(params->utils->conn,
+						SASL_CB_USER,
+						&getuser_cb,
+						&getuser_context);
+	    switch (result) {
+	    case SASL_INTERACT:
+		/* We'll set up an interaction later. */
+		prompt_for_userid = 1;
+		break;
+	    case SASL_OK:
+		if (! getuser_cb)
+		    break;
+		result = getuser_cb(getuser_context,
+				    SASL_CB_USER,
+				    &userid,
+				    NULL);
+		if (result != SASL_OK)
+		    return result;
+		if (userid) {
+		    oparams->user = params->utils->malloc(strlen(userid) + 1);
+		    if (! oparams->user)
+			return SASL_NOMEM;
+		    strcpy(oparams->user, userid);
+		}
+		break;
+	    default:
+		return result;
+	    }
+	}
+      
+	/* And now, if we *still* don't have userid,
+	 * but we think we can prompt, we need to set up a prompt. */
+	if (! oparams->user && prompt_for_userid) {
+	    if (! prompt_need)
+		return SASL_INTERACT;
+	    *prompt_need = params->utils->malloc(sizeof(sasl_interact_t) * 2);
+	    if (! *prompt_need)
+		return SASL_NOMEM;
+	    prompt = *prompt_need;
+	    prompt->id = SASL_CB_USER;
+	    prompt->prompt = "Remote Userid";
+	    prompt->defresult = NULL;
+	    prompt++;
+	    prompt->id = SASL_CB_LIST_END;
+	    return SASL_INTERACT;
+	}
+      
+	/* must be 8 octets */
+	if (serverinlen!=8) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", SASL_BADAUTH,
+			       0, "server response not 8 bytes long");
+	    return SASL_BADAUTH;
+	}
+
+	memcpy(in, serverin, 8);
+
+	/* get credentials */
+	if ((result = krb_get_cred((char *)params->service,
+			  text->instance,
+			  text->realm,
+			  &text->credentials)) != 0) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADAUTH, 0, 
+			       "krb_get_cred() failed: %s (%d)",
+			       krb_err_txt[result], result);
+	    return SASL_BADAUTH;
+	}
+
+	memcpy(text->session, text->credentials.session, 8);
+
+	/* make key schedule for encryption and decryption */
+	des_key_sched(&text->session, text->init_keysched);
+	des_key_sched(&text->session, text->enc_keysched);
+	des_key_sched(&text->session, text->dec_keysched);
+
+	/* decrypt from server */
+	des_ecb_encrypt((des_cblock *)in, (des_cblock *)in,
+			text->init_keysched, DES_DECRYPT);
+
+	/* convert to 32bit int */
+	testnum = (in[0]*256*256*256)+(in[1]*256*256)+(in[2]*256)+in[3];
+
+	/* verify data 1st 4 octets must be equal to chal+1 */
+	if (testnum != text->challenge+1)
+	{
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADAUTH, 0, 
+			       "server response incorrect");
+	    return SASL_BADAUTH;
+	}
+
+	/* construct 8 octets
+	 * 1-4 - original checksum
+	 * 5 - bitmask of sec layer
+	 * 6-8 max buffer size
+	 */
+	if (params->props.min_ssf > 
+	       KRB_DES_SECURITY_BITS + params->external_ssf) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_TOOWEAK, 0, 
+			       "minimum ssf too strong for this mechanism");
+	    return SASL_TOOWEAK;
+	} else if (params->props.min_ssf > params->props.max_ssf) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADPARAM, 0, 
+			       "minimum ssf larger than maximum ssf");
+	    return SASL_BADPARAM;
+	}
+
+	/* create stuff to send to server */
+	sout = (char *) params->utils->malloc(9+strlen(oparams->user)+9);
+
+	nchal=htonl(text->challenge);
+	memcpy(sout, &nchal, 4);
+
+	/* need bits of layer */
+	need = params->props.max_ssf - params->external_ssf;
+	musthave = params->props.min_ssf - params->external_ssf;
+
+	if ((in[4] & KRB_SECFLAG_ENCRYPTION)
+	    && (need>=56) && (musthave <= 56)) {
+	    /* encryption */
+	    oparams->encode = &privacy_encode;
+	    oparams->decode = &privacy_decode;
+	    oparams->mech_ssf = 56;
+	    sout[4] = KRB_SECFLAG_ENCRYPTION;
+	    /* using encryption layer */
+	} else if ((in[4] & KRB_SECFLAG_INTEGRITY)
+		   && (need >= 1) && (musthave <= 1)) {
+	    /* integrity */
+	    oparams->encode=&integrity_encode;
+	    oparams->decode=&integrity_decode;
+	    oparams->mech_ssf=1;
+	    sout[4] = KRB_SECFLAG_INTEGRITY;
+	    /* using integrity layer */
+	} else if ((in[4] & KRB_SECFLAG_NONE) && (musthave <= 0)) {
+	    /* no layer */
+	    oparams->encode=NULL;
+	    oparams->decode=NULL;
+	    oparams->mech_ssf=0;
+	    sout[4] = KRB_SECFLAG_NONE;
+	} else {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       SASL_BADPROT, 0, 
+			       "unable to agree on layers with server");
+	    return SASL_BADPROT;
+	}
+
+	servermaxbuf=in[5]*256*256+in[6]*256+in[7];
+	oparams->maxoutbuf=servermaxbuf;
+
+	sout[5] = (oparams->maxoutbuf) >> 16;  /* max ciphertext buffer size */
+	sout[6] = (oparams->maxoutbuf) >> 8;
+	sout[7] = (oparams->maxoutbuf);
+
+	sout[8] = 0x00; /* just to be safe */
+
+	/* append userid */
+	len = 9;			/* 8 + trailing NULL */
+	if (oparams->user) {
+	    strcpy((char *)sout + 8, oparams->user);
+	    len += strlen(oparams->user);
+	}
+
+	/* append 0 based octets so is multiple of 8 */
+	while(len % 8)
+	{
+	    sout[len]=0;
+	    len++;
+	}
+	sout[len]=0;
+    
+	des_pcbc_encrypt((des_cblock *)sout,
+			 (des_cblock *)sout,
+			 len,
+			 text->init_keysched,
+			 (des_cblock *)text->session,
+			 DES_ENCRYPT);
+
+	*clientout = params->utils->malloc(len);
+	memcpy((char *) *clientout, sout, len);
+
+	*clientoutlen=len;
+
+	/* nothing more to do; should be authenticated */
+	result = params->utils->getprop(params->utils->conn,
+					SASL_IP_LOCAL, 
+					(void **)&(text->ip_local));
+	if (result != SASL_OK) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       result, 0, 
+			       "unable to get local IP address: %z");
+	    return result;
+	}
+
+	result = params->utils->getprop(params->utils->conn,
+					SASL_IP_REMOTE, 
+					(void **)&(text->ip_remote));
+	if (result != SASL_OK) {
+	    params->utils->log(NULL, SASL_LOG_ERR, "KERBEROS_V4", 
+			       result, 0, 
+			       "unable to get remote IP address: %z");
+	    return result;
+	}
+
+	oparams->authid =
+	    params->utils->malloc(strlen(text->credentials.pname)
+				  + strlen(text->credentials.pinst)
+				  + 2);
+	if (! oparams->authid)
+	    return SASL_NOMEM;
+	strcpy(oparams->authid, text->credentials.pname);
+	if (text->credentials.pinst[0]) {
+	    strcat(oparams->authid, ".");
+	    strcat(oparams->authid, text->credentials.pinst);
+	}
+
+	if (oparams->user && !oparams->user[0]) {
+	    params->utils->free(oparams->user);
+	    oparams->user = NULL;
+	}
+	if (! oparams->user) {
+	    oparams->user = params->utils->malloc(strlen(oparams->authid) + 1);
+	    if (! oparams->user)
+		return SASL_NOMEM;
+	    strcpy(oparams->user, oparams->authid);
+	}
+
+	oparams->doneflag=1;
+	oparams->param_version=0;
+
+	text->size=-1;
+	text->needsize=4;
+
+	text->state++;
+
+	if (sout) params->utils->free(sout);
+
+	return SASL_CONTINUE;
+    }
+    else if (text->state==3) {
+	*clientout = NULL;
+	*clientoutlen = 0;
+
+	/* we're done! */
+	text->state++;
+	return SASL_OK;
     }
 
-    oparams->authid =
-      params->utils->malloc(strlen(text->credentials.pname)
-			    + strlen(text->credentials.pinst)
-			    + 2);
-    if (! oparams->authid)
-      return SASL_NOMEM;
-    strcpy(oparams->authid, text->credentials.pname);
-    if (text->credentials.pinst[0]) {
-      strcat(oparams->authid, ".");
-      strcat(oparams->authid, text->credentials.pinst);
-    }
-
-    if (oparams->user && !oparams->user[0]) {
-      params->utils->free(oparams->user);
-      oparams->user = NULL;
-    }
-    if (! oparams->user) {
-      oparams->user = params->utils->malloc(strlen(oparams->authid) + 1);
-      if (! oparams->user)
-	return SASL_NOMEM;
-      strcpy(oparams->user, oparams->authid);
-    }
-
-    oparams->doneflag=1;
-    oparams->param_version=0;
-
-    text->size=-1;
-    text->needsize=4;
-
-    text->state++;
-
-    return SASL_OK;
-  }
-
-  return SASL_FAIL; /* should never get here */
+    return SASL_FAIL; /* should never get here */
 }
 
 static const long client_required_prompts[] = {
