@@ -1,40 +1,50 @@
 /* Plain SASL plugin
  * Tim Martin 
- * $Id: plain.c,v 1.37 2000/02/23 01:16:14 tmartin Exp $
+ * $Id: plain.c,v 1.43 2001/02/19 19:01:54 leg Exp $
  */
-/***********************************************************
-        Copyright 1998 by Carnegie Mellon University
 
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of CMU not be
-used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.
-
-CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-******************************************************************/
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
 #include <config.h>
 #include <stdio.h>
-#ifndef SASL_MINIMAL_SERVER
-#include <pwd.h>
-#endif /* SASL_MINIMAL_SERVER */
-#ifdef HAVE_CRYPT_H
-#include <crypt.h>
-#endif /* HAVE_CRYPT_H */
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif /* HAVE_SHADOW_H */
 #include <sasl.h>
 #include <saslplug.h>
 
@@ -120,26 +130,11 @@ static void mech_free(void *global_context, sasl_utils_t *utils)
   utils->free(global_context);  
 }
 
-#ifdef SASL_MINIMAL_SERVER
-
-static int server_continue_step (void *conn_context,
-	      sasl_server_params_t *params,
-	      const char *clientin,
-	      int clientinlen,
-	      char **serverout,
-	      int *serveroutlen,
-	      sasl_out_params_t *oparams,
-	      const char **errstr)
-{
-  return SASL_FAIL;
-}
-
-#else /* SASL_MINIMAL_SERVER */
-
 /* fills in password; remember to free password and wipe it out correctly */
 static
 int verify_password(sasl_server_params_t *params, 
-		    const char *user, const char *pass)
+		    const char *user, const char *pass,
+		    const char **errstr)
 {
     const char *mech = NULL;
     int result;
@@ -147,22 +142,10 @@ int verify_password(sasl_server_params_t *params,
     params->utils->getopt(params->utils->getopt_context, "PLAIN",
 			  "pwcheck_method", &mech, NULL);
 
-    if (mech == NULL) {
-#ifdef HAVE_PAM
-	mech = "PAM";
-#else
-#ifdef HAVE_KRB
-	mech = "kerberos_v4";
-#else
-	mech = "passwd";
-#endif /* HAVE_KRB */
-#endif /* HAVE_PAM */
-    }
-
-    VL(("Checking password with %s mechanism\n",mech));
-
+    /* if it's null, checkpass will default */
     result = params->utils->checkpass(params->utils->conn,
-				      mech, params->service, user, pass);
+				      mech, params->service, 
+				      user, pass, errstr);
     
     return result;
 }
@@ -180,8 +163,7 @@ server_continue_step (void *conn_context,
   context_t *text;
   text=conn_context;
 
-  if (errstr)
-    *errstr = NULL;
+  if (errstr) { *errstr = NULL; }
 
   oparams->mech_ssf=0;
 
@@ -268,7 +250,7 @@ server_continue_step (void *conn_context,
     passcopy[password_len] = '\0';
 
     /* verify password - return sasl_ok on success*/    
-    result = verify_password(params, authen, passcopy);
+    result = verify_password(params, authen, passcopy, errstr);
     
     for (lup = strlen(passcopy); lup >= 0; lup--) {
 	passcopy[lup] = '\0';
@@ -321,7 +303,6 @@ server_continue_step (void *conn_context,
 
   return SASL_FAIL; /* should never get here */
 }
-#endif /* SASL_MINIMAL_SERVER */
 
 static const sasl_server_plug_t plugins[] = 
 {
@@ -416,12 +397,12 @@ static int get_userid(sasl_client_params_t *params,
   prompt=find_prompt(prompt_need,SASL_CB_USER);
   if (prompt!=NULL)
     {
-      /* copy it */
-      *userid=params->utils->malloc(strlen(prompt->result)+1);
-      if ((*userid)==NULL) return SASL_NOMEM;
+	/* copy it */
+	*userid=params->utils->malloc(prompt->len+1);
+	if ((*userid)==NULL) return SASL_NOMEM;
 	
-      strcpy(*userid, prompt->result);
-      return SASL_OK;
+	strncpy(*userid, prompt->result, prompt->len+1);
+	return SASL_OK;
     }
 
   /* Try to get the callback... */
@@ -463,12 +444,12 @@ static int get_authid(sasl_client_params_t *params,
   prompt=find_prompt(prompt_need,SASL_CB_AUTHNAME);
   if (prompt!=NULL)
   {
-    /* copy it */
-    *authid=params->utils->malloc(strlen(prompt->result)+1);
-    if ((*authid)==NULL) return SASL_NOMEM;
-
-    strcpy(*authid, prompt->result);
-    return SASL_OK;
+      /* copy it */
+      *authid=params->utils->malloc(prompt->len+1);
+      if ((*authid)==NULL) return SASL_NOMEM;
+      
+      strncpy(*authid, prompt->result, prompt->len+1);    
+      return SASL_OK;
   }
 
   /* Try to get the callback... */
@@ -646,7 +627,14 @@ static int client_continue_step (void *conn_context,
   context_t *text;
   text=conn_context;
 
-  VL(("Plain step #%i\n",text->state));
+  VL(("Plain step #%d\n",text->state));
+
+  if (!clientout && text->state == 1) {
+      text->state = 2;
+      return SASL_CONTINUE;
+  }
+  *clientout = NULL;
+  *clientoutlen = 0;
 
   /* doesn't really matter how the server responds */
 
@@ -772,9 +760,18 @@ static int client_continue_step (void *conn_context,
 
     oparams->param_version = 0;
 
-    text->state++;
+    text->state = 3;
 
-    return SASL_OK;
+    return SASL_CONTINUE;
+  }
+
+  if (text->state == 3)
+  {
+      *clientout = NULL;
+      *clientoutlen = 0;
+      VL(("Verify we're done step"));
+      text->state++;
+      return SASL_OK;      
   }
 
   return SASL_FAIL; /* should never get here */
