@@ -1,7 +1,75 @@
+/* testsuite.c -- Stress the library a little
+ * Tim Martin
+ */
+/* 
+ * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+
+/*
+ * To create a krb5 srvtab file given a krb4 srvtab
+ *
+ * ~/> ktutil
+ * ktutil:  rst /etc/srvtab
+ * ktutil:  wkt /etc/krb5.keytab
+ * ktutil:  q
+ */
+
+/*
+ * TODO:
+ *  put in alloc() routines that fail occasionally.
+ *  verify ssf's
+ */
+
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <sasl.h>
 
+#include <sasl.h>
+#include <saslutil.h>
+
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <time.h>
+#include <string.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <sys/socket.h>
@@ -14,23 +82,29 @@ char myhostname[1024+1];
 
 #define CLIENT_TO_SERVER "Hello. Here is some stuff"
 
+#define REALLY_LONG_LENGTH  32000
+#define REALLY_LONG_BACKOFF  2000
+
 char *username = "tmartin";
 char *authname = "tmartin";
 char *password = "1234";
 
+static char *gssapi_service = "host";
+
 /* our types of failures */
 typedef enum {
     NOTHING = 0,
-    ONEBYTE_RANDOM, /* replace one byte with something random */
-    ONEBYTE_NULL,   /* replace one byte with a null */
-    ONEBYTE_QUOTES, /* replace one byte with a double quote (try to fuck with digest-md5) */   
-    ONLY_ONE_BYTE,  /* send only one byte */
-    ADDSOME,        /* add some random bytes onto the end */
-    SHORTEN,        /* shorten the string some */
-    REASONABLE_RANDOM, /* send same size but random */
-    REALLYBIG,       /* send something absurdly large (all random)*/
-    NEGATIVE_LENGTH, /* send negative length */
-    CORRUPT_SIZE /* keep this one last */
+    ONEBYTE_RANDOM,		/* replace one byte with something random */
+    ONEBYTE_NULL,		/* replace one byte with a null */
+    ONEBYTE_QUOTES,		/* replace one byte with a double quote 
+				   (try to fuck with digest-md5) */
+    ONLY_ONE_BYTE,		/* send only one byte */
+    ADDSOME,			/* add some random bytes onto the end */
+    SHORTEN,			/* shorten the string some */
+    REASONABLE_RANDOM,		/* send same size but random */
+    REALLYBIG,			/* send something absurdly large all random */
+    NEGATIVE_LENGTH,		/* send negative length */
+    CORRUPT_SIZE		/* keep this one last */
 } corrupt_type_t;
 
 typedef void *foreach_t(char *mech, void *rock);
@@ -42,19 +116,74 @@ typedef struct tosend_s {
 
 } tosend_t;
 
-static struct sasl_callback nullsasl_cb[] = {
-    { SASL_CB_GETOPT, NULL, NULL },
-    { SASL_CB_LIST_END, NULL, NULL }
-};
+
+void fatal(char *str)
+{
+    printf("Failed with: %s\n",str);
+    exit(3);
+}
+
+
+
+/* my mutex functions */
+int g_mutex_cnt = 0;
+
+typedef struct my_mutex_s {
+
+    int num;
+    int val;
+    
+} my_mutex_t;
+
+void *my_mutex_new(void)
+{
+    my_mutex_t *ret = (my_mutex_t *)malloc(sizeof(my_mutex_t));
+    ret->num = g_mutex_cnt;
+    g_mutex_cnt++;
+
+    ret->val = 0;
+
+    return ret;
+}
+
+int my_mutex_lock(my_mutex_t *m)
+{
+    if (m->val != 0)
+    {
+	fatal("Trying to lock a mutex already locked. This is not good in a single threaded app");
+    }
+
+    m->val = 1;
+    return SASL_OK;
+}
+
+int my_mutex_unlock(my_mutex_t *m)
+{
+    if (m->val != 1)
+    {
+	fatal("Unlocking mutex that isn't locked");
+    }
+
+    m->val = 0;
+
+    return SASL_OK;
+}
+
+int my_mutex_dispose(my_mutex_t *m)
+{
+    if (m==NULL) return SASL_OK;
+
+    free(m);
+
+    return SASL_OK;
+}
 
 int good_getopt(void *context __attribute__((unused)), 
-		const char *plugin_name,
+		const char *plugin_name __attribute__((unused)), 
 		const char *option,
 		const char **result,
 		unsigned *len)
 {
-    char opt[1024];
-
     if (strcmp(option,"pwcheck_method")==0)
     {
 	*result = "sasldb";
@@ -71,7 +200,7 @@ static struct sasl_callback goodsasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-int givebadpath(void * context,
+int givebadpath(void * context __attribute__((unused)), 
 		char ** path)
 {
     int lup;
@@ -89,7 +218,7 @@ static struct sasl_callback withbadpathsasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-int giveokpath(void * context,
+int giveokpath(void * context __attribute__((unused)), 
 		char ** path)
 {
     *path = malloc(1000);
@@ -107,31 +236,30 @@ static struct sasl_callback emptysasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-void fatal(char *str)
-{
-    printf("Failed with: %s\n",str);
-    exit(3);
-}
-
-char really_long_string[32000];
+char really_long_string[REALLY_LONG_LENGTH];
 
 /*
  * Setup some things for test
  */
-void init(void)
+void init(unsigned int seed)
 {
     int lup;
     int result;
 
-    srand(time(NULL));
+    srand(seed);    
 
-    for (lup=0;lup<32000;lup++)
+    for (lup=0;lup<REALLY_LONG_LENGTH;lup++)
 	really_long_string[lup] = '0' + (rand() % 10);
 
-    really_long_string[30000+ rand() % 2000] = '\0';
+    really_long_string[REALLY_LONG_LENGTH - rand() % REALLY_LONG_BACKOFF] = '\0';
 
     result = gethostname(myhostname, sizeof(myhostname)-1);
     if (result == -1) fatal("gethostname");
+
+    sasl_set_mutex((sasl_mutex_new_t *) &my_mutex_new,
+		   (sasl_mutex_lock_t *) &my_mutex_lock,
+		   (sasl_mutex_unlock_t *) &my_mutex_unlock,
+		   (sasl_mutex_dispose_t *) &my_mutex_dispose);
 }
 
 /*
@@ -176,14 +304,14 @@ void test_init(void)
  * Tests sasl_listmech command
  */
 
-int test_listmech(void)
+void test_listmech(void)
 {
     sasl_conn_t *saslconn;
     int result;
-    char *str;
-    int plen;
+    char *str = NULL;
+    unsigned int plen;
     int lup;
-    int pcount;
+    unsigned int pcount;
 
     /* test without initializing library */
     result = sasl_listmech(NULL, /* conn */
@@ -224,6 +352,7 @@ int test_listmech(void)
     if (result != SASL_OK) fatal("Failed sasl_listmech() with long user");
 
     if (str[0]!='[') fatal("Failed sasl_listmech() with long user (didn't start with '['");
+    free(str);
 
     /* Test with really long prefix */
 
@@ -239,6 +368,7 @@ int test_listmech(void)
     if (result != SASL_OK) fatal("failed sasl_listmech() with long prefix");
 
     if (str[0]!=really_long_string[0]) fatal("failed sasl_listmech() with long prefix (str is suspect)");
+    free(str);
 
     /* Test with really long suffix */
 
@@ -252,6 +382,7 @@ int test_listmech(void)
 			   NULL);
 
     if (result != SASL_OK) fatal("Failed sasl_listmech() with long suffix");
+    free(str);
 
     /* Test with really long seperator */
 
@@ -265,6 +396,7 @@ int test_listmech(void)
 			   NULL);
 
     if (result != SASL_OK) fatal("Failed sasl_listmech() with long seperator");
+    free(str);
 
     /* Test contents of output string is accurate */
     result = sasl_listmech(saslconn,
@@ -278,7 +410,7 @@ int test_listmech(void)
 
     if (result != SASL_OK) fatal("Failed sasl_listmech()");
 
-    if (strlen(str)!=plen) fatal("Length of string doesn't match what we were told");
+    if ((int) strlen(str)!=plen) fatal("Length of string doesn't match what we were told");
     
     for (lup=0;lup<plen;lup++)
 	if (str[lup]=='%')
@@ -292,7 +424,9 @@ int test_listmech(void)
 	fatal("Number of mechs received doesn't match what we were told");
     }
 
+    free(str);
     /* Call sasl done then make sure listmech doesn't work anymore */
+    sasl_dispose(&saslconn);
     sasl_done();
 
     result = sasl_listmech(saslconn,
@@ -348,12 +482,14 @@ void test_random(void)
     /* try seeding with a lot of crap */
     sasl_randcreate(&rpool);
     
-    for (lup=0;lup<sizeof(buf);lup++)
+    for (lup=0;lup<(int) sizeof(buf);lup++)
     {
 	buf[lup] = (rand() % 256);	
     }
     sasl_randseed(rpool, buf, sizeof(buf));
     sasl_churn(rpool, buf, sizeof(buf));
+
+    sasl_randfree(&rpool);
 }
 
 /*
@@ -364,11 +500,11 @@ void test_64(void)
 {
     char orig[4096];
     char enc[8192];
-    int encsize;
+    unsigned encsize;
     int lup;
 
     /* make random crap and see if enc->dec produces same as original */
-    for (lup=0;lup<sizeof(orig);lup++)
+    for (lup=0;lup<(int) sizeof(orig);lup++)
 	orig[lup] = (char) (rand() % 256);
     
     if (sasl_encode64(orig, sizeof(orig), enc, sizeof(enc), &encsize)!=SASL_OK) 
@@ -378,7 +514,8 @@ void test_64(void)
 	fatal("decode failed when didn't expect");
     
     if (encsize != sizeof(orig)) fatal("Now has different size");
-    for (lup=0;lup<sizeof(orig);lup++)
+    
+    for (lup=0;lup<(int) sizeof(orig);lup++)
 	if (enc[lup] != orig[lup])
 	    fatal("enc64->dec64 doesn't match");
 
@@ -451,6 +588,7 @@ void interaction (int id, const char *prompt,
 
     *tlen = strlen(result);
     *tresult = (char *) malloc(*tlen+1);
+
     memset(*tresult, 0, *tlen+1);
     memcpy((char *) *tresult, result, *tlen);
 }
@@ -493,7 +631,7 @@ void set_properties(sasl_conn_t *conn, char *serverFQDN, int port)
  * This corrupts the string for us
  */
 
-void corrupt(corrupt_type_t type, char *in, int inlen, char **out, int *outlen)
+void corrupt(corrupt_type_t type, char *in, int inlen, char **out, unsigned *outlen)
 {
     int lup;
 
@@ -527,6 +665,7 @@ void corrupt(corrupt_type_t type, char *in, int inlen, char **out, int *outlen)
 	    *outlen = inlen;
 	    break;
 	case ONLY_ONE_BYTE:
+	    free(in);
 	    *out = (char *) malloc(1);
 	    (*out)[0] = (char) (rand() % 256);
 	    *outlen = 1;
@@ -539,18 +678,28 @@ void corrupt(corrupt_type_t type, char *in, int inlen, char **out, int *outlen)
 	    
 	    for (lup=inlen;lup<*outlen;lup++)
 		(*out)[lup] = (char) (rand() %256);
+
+	    free(in);
 	    break;
 
 	case SHORTEN:
-	    *outlen = (rand() % inlen);
-	    *out = (char *) malloc(*outlen);
-	    memcpy(*out, in, *outlen);
+	    if (inlen > 0)
+	    {
+		*outlen = (rand() % inlen);
+		*out = (char *) malloc(*outlen);
+		memcpy(*out, in, *outlen);
+		free(in);
+	    } else {
+		*outlen = inlen;
+		*out = in;
+	    }
 	    break;
 	case REASONABLE_RANDOM:
 	    *outlen = inlen;
 	    *out = (char *) malloc(*outlen);
 	    for (lup=0;lup<*outlen;lup++)
 		(*out)[lup] = (char) (rand() % 256);
+	    free(in);
 	    break;
 	case REALLYBIG:
 	    *outlen = rand() % 50000;
@@ -559,11 +708,13 @@ void corrupt(corrupt_type_t type, char *in, int inlen, char **out, int *outlen)
 	    for (lup=0;lup<*outlen;lup++)
 		(*out)[lup] = (char) (rand() % 256);
 	    
+	    free(in);
 	    break;
 	case NEGATIVE_LENGTH:
 
 	    *out = in;
-	    *outlen = -1* (rand() % inlen);
+	    if (inlen == 0) inlen = 10;
+	    *outlen = -1 * (rand() % inlen);
 	    
 	    break;
 	default:
@@ -577,20 +728,21 @@ void sendbadsecond(char *mech, void *rock)
     int result;
     sasl_conn_t *saslconn;
     sasl_conn_t *clientconn;
-    char *out, *dec;
-    unsigned outlen, declen;
+    char *out, *dec = NULL, *out2;
+    unsigned outlen, declen, outlen2;
     sasl_interact_t *client_interact=NULL;
     const char *mechusing;
     char *service = "rcmd";
     const char *errstr;
     int mystep = 0; /* what step in the authentication are we on */
     int mayfail = 0; /* we did some corruption earlier so it's likely to fail now */
+    char *tofree;
     
     tosend_t *send = (tosend_t *)rock;
 
     printf("%s --> start\n",mech);
     
-    if (strcmp(mech,"GSSAPI")==0) service = "imap";
+    if (strcmp(mech,"GSSAPI")==0) service = gssapi_service;
 
     if (sasl_client_init(client_callbacks)!=SASL_OK) fatal("Unable to init client");
     if (sasl_server_init(goodsasl_cb,"TestSuite")!=SASL_OK) fatal("");
@@ -598,7 +750,7 @@ void sendbadsecond(char *mech, void *rock)
     /* client new connection */
     if (sasl_client_new(service,
 			myhostname,
-			"ANDREW.CMU.EDU",
+			NULL,
 			0,
 			&clientconn)!= SASL_OK) fatal("sasl_client_new() failure");
 
@@ -634,6 +786,8 @@ void sendbadsecond(char *mech, void *rock)
 	corrupt(send->type, out, outlen, &out, &outlen);
 	mayfail = 1;
     }
+
+    tofree = out;
     result = sasl_server_start(saslconn,
 			       mech,
 			       out,
@@ -646,8 +800,11 @@ void sendbadsecond(char *mech, void *rock)
     {
 	if (result >= 0)
 	    printf("WARNING: We did a corruption but it still worked\n");
-	else
-	    return;
+	else {
+	    if (tofree) free(tofree);
+	    if (out!=tofree) free(out);
+	    goto done;
+	}
     } else {
 	if (result < 0) 
 	{
@@ -656,7 +813,7 @@ void sendbadsecond(char *mech, void *rock)
 	    fatal("sasl_server_start() error");
 	}
     }
-
+    if (tofree) free(tofree);
     mystep++;
 
     while (result == SASL_CONTINUE) {
@@ -666,11 +823,12 @@ void sendbadsecond(char *mech, void *rock)
 	    corrupt(send->type, out, outlen, &out, &outlen);
 	    mayfail = 1;
 	}
+	tofree = out;
 	do {
 	    result = sasl_client_step(clientconn,
 				      out, outlen,
 				      &client_interact,
-				      &out, &outlen);
+				      &out2, &outlen2);
 	    
 	    if (result == SASL_INTERACT) fillin_correctly(client_interact);
 	} while (result == SASL_INTERACT);
@@ -679,8 +837,11 @@ void sendbadsecond(char *mech, void *rock)
 	{
 	    if (result >= 0)
 		printf("WARNING: We did a corruption but it still worked\n");
-	    else
-		return;
+	    else {
+		if (tofree) free(tofree);
+		if (out!=tofree) free(out);
+		goto done;
+	    }
 	} else {
 	    if (result < 0) 
 	    {
@@ -689,27 +850,33 @@ void sendbadsecond(char *mech, void *rock)
 		fatal("sasl_client_step() error");
 	    }
 	}
+	if (tofree) free(tofree);
+	out=out2;
+	outlen=outlen2;
 	mystep++;
 
-
 	if (mystep == send->step)
 	{
 	    corrupt(send->type, out, outlen, &out, &outlen);
 	    mayfail = 1;
 	}
+	tofree = out;
 	result = sasl_server_step(saslconn,
 				  out,
 				  outlen,
 				  &out,
 				  &outlen,
-				  NULL);
+				  &errstr);
 	
 	if (mayfail == 1)
 	{
 	    if (result >= 0)
 		printf("WARNING: We did a corruption but it still worked\n");
-	    else
-		return;
+	    else {
+		if (tofree) free(tofree);
+		if (out!=tofree) free(out);
+		goto done;
+	    }
 	} else {
 	    if (result < 0) 
 	    {
@@ -718,29 +885,33 @@ void sendbadsecond(char *mech, void *rock)
 		fatal("sasl_server_step() error");
 	    }
 	}
+	if (tofree) free(tofree);
 	mystep++;
 
-    } 
-			       
-    printf("%s --> %s\n",mech,sasl_errstring(result,NULL,NULL));
+    }
+
+    if (out) free(out);
 
     /* client to server */
-    
     result = sasl_encode(clientconn, CLIENT_TO_SERVER, strlen(CLIENT_TO_SERVER), &out, &outlen);
     if (result != SASL_OK) fatal("Error encoding");
-
     if (mystep == send->step)
     {
 	corrupt(send->type, out, outlen, &out, &outlen);
 	mayfail = 1;
     }
+    tofree = out;
+    dec = NULL;
     result = sasl_decode(saslconn, out, outlen, &dec, &declen);
     if (mayfail == 1)
     {
 	if (result >= 0)
 	    printf("WARNING: We did a corruption but it still worked\n");
-	else
-	    return;
+	else {
+	    if (out) free(out);
+	    if (dec) free(dec);
+	    goto done;
+	}
     } else {
 	if (result < 0) 
 	{
@@ -749,10 +920,16 @@ void sendbadsecond(char *mech, void *rock)
 	    fatal("sasl_decode() failure");
 	}
     }
+    if (out) free(out);
+    if (dec) free(dec);
     mystep++;
 
     /* no need to do other direction since symetric */
-    
+
+    printf("%s --> %s\n",mech,sasl_errstring(result,NULL,NULL));
+ done:
+    sasl_dispose(&clientconn);
+    sasl_dispose(&saslconn);
     sasl_done();
 }
 
@@ -762,7 +939,7 @@ void sendbadsecond(char *mech, void *rock)
 
 void foreach_mechanism(foreach_t *func, void *rock)
 {
-    char *str, *start;
+    char *str, *start, *tofree;
     sasl_conn_t *saslconn;
     int result;
 
@@ -784,6 +961,8 @@ void foreach_mechanism(foreach_t *func, void *rock)
 			   &str,
 			   NULL,
 			   NULL);
+    tofree = str;
+    sasl_dispose(&saslconn);
     sasl_done();
 
     /* call the function for each mechanism */
@@ -803,18 +982,17 @@ void foreach_mechanism(foreach_t *func, void *rock)
 
 	start = str;
     }
-    
+    free(tofree);
 }
 
-void test_serverstart(void)
+void test_serverstart(int steps)
 {
     int result;
     sasl_conn_t *saslconn;
     char *out;
     unsigned outlen;
     tosend_t tosend;
-    int lup, lup2;
-    char randdata[4096];
+    int lup;
 
     if (sasl_server_init(emptysasl_cb,"TestSuite")!=SASL_OK) fatal("");
 
@@ -858,16 +1036,19 @@ void test_serverstart(void)
 
     if (result == SASL_OK) fatal("Said ok to invalid mechanism");
 
+    sasl_dispose(&saslconn);
+    sasl_done();
+
     tosend.step = 500;
     printf("trying to do correctly\n");
     foreach_mechanism((foreach_t *) &sendbadsecond,&tosend);
 
-    for (lup=0;lup<100;lup++)
+    for (lup=0;lup<steps;lup++)
     {
 	tosend.type = rand() % CORRUPT_SIZE;
 	tosend.step = lup % MAX_STEPS;
 
-	printf("trying random crap (%d of %d)\n",lup,100);
+	printf("trying random crap (%d of %d)\n",lup,steps);
 	foreach_mechanism((foreach_t *) &sendbadsecond,&tosend);
     }
 }
@@ -911,19 +1092,28 @@ void create_ids(void)
     if (sasl_setpass(saslconn,username, password, 0, SASL_SET_CREATE, NULL)==SASL_OK)
 	fatal("Allowed password of zero length");
     if (sasl_setpass(saslconn,username, password, strlen(password), 43, NULL)==SASL_OK)
-	fatal("Gave wierd code");
+	fatal("Gave weird code");
 
+#ifndef SASL_NDBM
     if (sasl_setpass(saslconn,really_long_string, password, strlen(password), 
 		     SASL_SET_CREATE, NULL)!=SASL_OK)
 	fatal("Didn't allow really long username");
+#else
+    printf("WARNING: skipping sasl_setpass() on really_long_string with NDBM\n");
+#endif
 
     if (sasl_setpass(saslconn,"bob" ,really_long_string, strlen(really_long_string), 
 		     SASL_SET_CREATE, NULL)!=SASL_OK)
 	fatal("Didn't allow really long password");
 
-    if (sasl_setpass(saslconn,"frank" ,password, strlen(password), 
-		     SASL_SET_DISABLE, NULL)!=SASL_NOUSER)
-	fatal("Disabling non-existant didn't return SASL_NOUSER");
+    result = sasl_setpass(saslconn,"frank" ,password, strlen(password), 
+		     SASL_SET_DISABLE, NULL);
+
+    if ((result!=SASL_NOUSER) && (result!=SASL_OK))
+	{
+	    printf("error = %d\n",result);
+	    fatal("Disabling non-existant didn't return SASL_NOUSER");
+	}
     
 
     /* Now set the user again (we use for rest of program) */
@@ -931,8 +1121,57 @@ void create_ids(void)
     if (result != SASL_OK)
 	fatal("Error setting password. Do we have write access to sasldb?");
 
+    /* cleanup */
+    sasl_dispose(&saslconn);
+    sasl_done();
 }
 
+/*
+ * Test the checkpass routine
+ */
+
+void test_checkpass(void)
+{
+    sasl_conn_t *saslconn;
+
+    /* try without initializing anything */
+    sasl_checkpass(NULL, username, strlen(username),
+		   password, strlen(password), NULL);
+
+    if (sasl_server_init(goodsasl_cb,"TestSuite")!=SASL_OK) fatal("");
+
+    if (sasl_server_new("rcmd", myhostname,
+			NULL, NULL, SASL_SECURITY_LAYER, 
+			&saslconn) != SASL_OK)
+	fatal("");
+
+    /* make sure works for general case */
+
+    if (sasl_checkpass(saslconn, username, strlen(username),
+		       password, strlen(password), NULL)!=SASL_OK)
+	fatal("sasl_checkpass() failed on simple case");
+
+    /* NULL saslconn */
+    if (sasl_checkpass(NULL, username, strlen(username),
+		   password, strlen(password), NULL) == SASL_OK)
+	fatal("Suceeded with NULL saslconn");
+
+    /* NULL username */
+    if (sasl_checkpass(saslconn, NULL, strlen(username),
+		   password, strlen(password), NULL) == SASL_OK)
+	fatal("Suceeded with NULL username");
+
+    /* NULL password */
+    if (sasl_checkpass(saslconn, username, strlen(username),
+		   NULL, strlen(password), NULL) == SASL_OK)
+	fatal("Suceeded with NULL password");
+
+    sasl_dispose(&saslconn);
+    sasl_done();
+}
+
+
+
 void notes(void)
 {
     printf("NOTE:\n");
@@ -942,16 +1181,49 @@ void notes(void)
     printf("\n\n");
 }
 
-
-int main()
+void usage(void)
 {
+    printf("Usage:\n" \
+           " testsuite [-g][-s]\n" \
+           "    g -- gssapi service name to use (default: host)\n" \
+	   "    h -- show this screen\n" \
+           "    s -- random seed to use\n" \
+           );
+}
+
+int main(int argc, char **argv)
+{
+    char c;
+    unsigned int seed = time(NULL);
+    char *a;
+    while ((c = getopt(argc, argv, "s:g:h:")) != EOF)
+	switch (c) {
+	case 's':
+	    seed = atoi(optarg);
+	    break;
+	case 'g':
+	    gssapi_service = optarg;
+	    break;
+	case 'h':
+	    usage();
+	    exit(0);
+	    break;
+	default:
+	    usage();
+	    fatal("Invalid parameter\n");
+	    break;
+    }
+
     notes();
 
-    init();
+    init(seed);
 
     create_ids();
     printf("Created id's in sasldb... ok\n");
 
+    test_checkpass();
+    printf("Checking plaintext passwords... ok\n");
+
     test_random();
     printf("Random number functions... ok\n");
 
@@ -963,10 +1235,12 @@ int main()
 
     test_listmech();
     printf("Tests of sasl_listmech()... ok\n");
+    
 
-
-    test_serverstart();
+    test_serverstart(7);
     printf("Tests of sasl_server_start()... ok\n");
 
     printf("All tests seemed to go ok\n");
+
+    exit(0);
 }
